##
## This example was mostly taken from NVIDIA's Cg demo collection.
##
## Charles Mason <cmason@cs.fsu.edu>
##
## Hacked by Calle Lejdfors <calle.lejdfors@cs.lth.se>
##
##
##

###############################
#
# Imports
#
###############################
import sys
import string
import math
import PIL.Image
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from PIL import *
sys.path.append('lib')
import FX 
from OpenGL.GL.ARB.multitexture import *
sys.path.append('effects')
from glut_extra import *
from tex_object import *
import bunny

###############################
#
# Global Variables
#
###############################
bunnyList = 0
effects = {}
effect = None
currObj = 1

#### Extra GLUT stuff ####
cb = glut_callbacks()
reshaper = glut_perspective_reshaper()
camera = glut_simple_mouse_interactor()
object = glut_simple_mouse_interactor()
cubemap = tex_object_cube_map()


#Shader.basePath = 'effects/programs'
#Texture.basePath = 'effects/textures'

#supportsMultitex = glInitMultiTexARB()

###############################
#
# Code
#
###############################
def main():
	""" main program
	"""

	global cb, reshaper

	glutInit(sys.argv)
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH)
	glutInitWindowSize( 512, 512 )
	glutInitWindowPosition( 256, 256 )
	glutCreateWindow( "test" )
	
	# Load up the texture, etc
	init_opengl()

	# Prepare some extra glut stuff
	glut_helpers_initialize()

	cb.keyboard_function = keyboard;
	cb.idle_function = idle;
	camera.configure_buttons(1)
	camera.set_camera_mode(1)
	camera.pan.pan = [ 0.0, 0.0, 3.0 ]
	object.configure_buttons(1);
	object.dolly.dolly[2] = -1;

	glut_add_interactor(cb)
	glut_add_interactor(object)
	glut_add_interactor(reshaper)
	
	glut_idle(1)

	# Callbacks
	glutDisplayFunc( display )
	glutMainLoop()


def cleanExit(exitval=0):
	"""exits the program with exit code 'exitval'"""
	sys.exit(exitval)

def init_opengl():
	"""prepare the OpenGL state"""

	global effects,effect,skyBoxEffect,supportsMultitex 

        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)

	glEnable(GL_DEPTH_TEST)

	FX.State._uglyINIT()
	
	#FX.Shader.basePath = 'effects/programs'
	FX.Texture.basePath = 'effects/textures'

	supportsMultitex = glInitMultiTexARB()

	Effect = FX.Effect
	
        effects['f'] = Effect('Fresnel')
        effects['l'] = Effect('Leather')
        effects['g'] = Effect('Gouraud')
        effects['p'] = Effect('Phong')
        effects['t'] = Effect('Toon')
        effects['r'] = Effect('Refract')
        effects['s'] = Effect('Skybox')
	effects['i'] = Effect('ImpureRefract')
	effects['d'] = Effect('DepthColor')
	##effects['b'] = Effect('Beer')
	##effects['u'] = Effect('BumpMapping')
	effects['u'] = Effect('Durer')
	
        print "Loaded effects:"
        #print " i: Simple"
        print " f: Fresnel"
        print " l: Leather"
        print " g: Gouraud"
        print " p: Phong"
        print " t: Toon"
        print " r: Refract"
        print " s: Skybox"
	print " i: ImpureRefract"
	print " d: DepthColor"
	##print " b: Beer"
	##print " u: Bumpmapping"
	print " u: Durer"
        

        skyBoxEffect = Effect('Skybox') 
        
	
	global bunnyList
	bunnyList = bunny.MakeBunnyList()
	

def drawSkyBox():
	for p in skyBoxEffect.passes():
                p()
		glutSolidSphere(5.0, 40, 20);

def draw_plane(w, h, rows, cols):

	ts = 1.0 / cols
	tt = 1.0 / rows
	tw = w / cols
	th = h / cols

	#glEnable(GL_COLOR_MATERIAL)
	glNormal3f(0.0, 0.0, 1.0)
	for y in range(rows):
		glBegin(GL_QUAD_STRIP)
		for x in range(cols+1):
			vx = tw * x - (w / 2.0)
			vy = th * y - (h / 2.0)
			s = ts * x
			t = tt * y

			if supportsMultitex:
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, s, t)
				glMultiTexCoord3fARB(GL_TEXTURE1_ARB, 1.0, 0.0, 0.0)
				glMultiTexCoord3fARB(GL_TEXTURE2_ARB, 0.0, 1.0, 0.0)
			else:
				glTexCoord2f(s,t)
			glColor3f(s, t, 0.0)
			glVertex3f(vx, vy, 0.0)

			if supportsMultitex:
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, s, t + tt)
				glMultiTexCoord3fARB(GL_TEXTURE1_ARB, 1.0, 0.0, 0.0)
				glMultiTexCoord3fARB(GL_TEXTURE2_ARB, 0.0, 1.0, 0.0)
			else:
				glTexCoord2f(s,t+tt)
			glColor3f(s, t + tt, 0.0)
			glVertex3f(vx, vy + th, 0.0)	
		glEnd()

def draw_obj(obj):
	if obj == 1:
            glutSolidSphere(0.5, 150, 150)
	elif obj == 2:
            glutSolidTorus(0.25, 0.5, 100, 100)
	elif obj == 3:
            draw_plane(1.0, 1.0, 10, 10)
	elif obj == 4:
            glCallList(bunnyList)

def display():
	"""display callback from glut.  Called whenever a render is needed."""

	# Prepare Scene
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	glMatrixMode( GL_MODELVIEW )
	glLoadIdentity()

	drawSkyBox()

	# I hope that the nvidia camera code is ported correctly.
	camera.apply_inverse_transform()
	object.apply_transform()

	glColor3f(1.0, 1.0, 1.0)

	glPushMatrix()
	glScalef(2, 2, 2)

	if effect:
		for p in effect.passes():
                        p()
			draw_obj(currObj);
	else:
		draw_obj(currObj);

	glPopMatrix()

	glutSwapBuffers();


def keyboard(key, x, y):
	"""key is a string of what key is pressed"""

	global currObj,effect

	idx = ord(key)

	if key == ' ':
            effect = None

        if idx == 27 or key == 'q':
            cleanExit()
            
	if ord(key) >= ord('1') and ord(key) <= ord('4'):
            currObj = ord(key)-ord('0')

	if key == 'w':
            # Wireframe toggle
            
            m = glGetIntegerv(GL_POLYGON_MODE)
            if m[1] == GL_FILL:
                glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
            elif m[1] == GL_LINE:
                glPointSize( 4 )
                glPolygonMode( GL_FRONT_AND_BACK, GL_POINT );
            else:
                glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

        if effects.has_key(key):
            effect = effects[key]

def idle():
	"""idle callback from glut"""
	object.trackball.increment_rotation()
	glutPostRedisplay()


def menu(entry): keyboard(chr(entry), 0, 0)

# Start of program
if __name__ == "__main__":
    main()




