from FX import *

lightPos = [0, 4, 10, 1]
ambientMaterial = [0.1, 0.1, 0.1]
diffuseMaterial = [1.0, 0.0, 0.0]
specularMaterial = [1.0, 1.0, 1.0]
shininess = 80
delta = 0.05

heightTex = Texture('heart_heightMap.png',
                    TEXTURE_2D)


cg = Cg( """
void vertex(float4 position   : POSITION,
            float4 normal     : NORMAL,
            float2 parameters : TEXCOORD0,
            float4 tangent    : TEXCOORD1,
            float4 binormal   : TEXCOORD2,

        out float4 clipPos     : POSITION,
        out float2 texCoord    : TEXCOORD0,
        out float4 eyePos      : TEXCOORD1,
        out float3 eyeNormal   : TEXCOORD2,
        out float3 eyeTangent  : TEXCOORD3,
        out float3 eyeBinormal : TEXCOORD4,
        
    uniform float4x4 ModelView,
    uniform float4x4 Projection,
    uniform float4x4 ModelViewIT)
{
    texCoord = parameters;

    eyePos = mul(ModelView, position);

    clipPos = mul(Projection, eyePos);
    eyeNormal = normalize(mul(ModelViewIT, normal).xyz);
    eyeTangent = normalize(mul(ModelViewIT, tangent).xyz);
    eyeBinormal = normalize(mul(ModelViewIT, binormal).xyz);
}

void fragment(float2 texCoord    : TEXCOORD0,
              float3 eyePos      : TEXCOORD1,
              float3 eyeNormal   : TEXCOORD2,
              float3 eyeTangent  : TEXCOORD3,
              float3 eyeBinormal : TEXCOORD4,

          out float3 color : COLOR,
          
      uniform float4    lightPos,
      uniform float3    ambientMaterial,
      uniform float3    diffuseMaterial,
      uniform float3    specularMaterial,
      uniform float     shininess,
      uniform float     delta,
      uniform sampler2D heightMap)
{
    float h = tex2D(heightMap, texCoord).r;
    float ha = tex2D(heightMap, texCoord + float2(0, delta)).r;
    float hr = tex2D(heightMap, texCoord + float2(delta, 0)).r;
    float3 mapNormal = normalize(float3(h - ha, h - hr, 1));
        
    float3x3 tangent2eye = transpose(float3x3(eyeTangent,
                                              eyeBinormal,
                                              eyeNormal));
    
    float3 normal = normalize(mul(tangent2eye, mapNormal));
    float3 lightVec = normalize(lightPos.xyz - eyePos);
    float3 eyeVec = -normalize(eyePos);
    float3 halfVec = (lightVec + eyeVec) / 2;

    float diffuse = dot(normal, lightVec);
    float specular = dot(normal, halfVec);
        
    float4 lighting = lit(diffuse, specular, shininess);
                
    color = (ambientMaterial + lighting.y * diffuseMaterial + 
             lighting.z * specularMaterial);
}""")

heightMap = Sampler(texture = heightTex,
                    WrapS = GL_CLAMP,
                    WrapT = GL_CLAMP,
                    MinFilter = GL_LINEAR,
                    MagFilter = GL_LINEAR)


technique = [Render(VertexShader = cg.vertex(target = arbvp1),
                    FragmentShader = cg.fragment(target = arbfp1))]

