# /*********************************************************************NVMH3****
# Path:  SDK\MEDIA\programs\cg_refract
# File:  cg_refract.cg
#
# Copyright NVIDIA Corporation 2002
# TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
# *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
# OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
# BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
# WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
# BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
# ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
# BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#
# Comments:
#
# ******************************************************************************/
#

#
# Style changes and PyFX porting done by Calle Lejdfors <calle.lejdfors@cs.lth.se>
#


from FX import *
import Skybox

theta = 0.9 
enableRefraction = 1.0
enableFresnel = 0.3


refract = Cg( """
// fresnel approximation
fixed fast_fresnel(float3 I, float3 N, float3 fresnelValues)
{
    fixed power = fresnelValues.x;
    fixed scale = fresnelValues.y;
    fixed bias = fresnelValues.z;

    return bias + pow(1.0 - dot(I, N), power) * scale;
}

void vertex(float4 Position : POSITION,
            float4 Normal   : NORMAL,
               
        out float4 hPosition   : POSITION,
        out float4 fresnelTerm : COLOR0,
        out float4 refractVec  : TEXCOORD0,
        out float4 reflectVec  : TEXCOORD1,
           
    uniform float4x4 ModelViewProj,
    uniform float4x4 ModelView,
    uniform float4x4 ModelViewIT,
    uniform float    theta)
{
    hPosition = mul(ModelViewProj, Position);

    // convert the position and normal into appropriate spaces
    float3 eyeToVert = mul(ModelView, Position).xyz;
    eyeToVert = normalize(eyeToVert);
	
    float3 normal = mul(ModelViewIT, Normal).xyz;
    normal = normalize(normal);

    // calculate refraction
    refractVec.xyz = refract(eyeToVert, normal, theta);
    refractVec.w = 1;

    // calculate reflection
    reflectVec.xyz = reflect(eyeToVert, normal);
    reflectVec.w = 1;

    // calculate fresnel term
    fresnelTerm = fast_fresnel(-eyeToVert, normal, float3(2.0, 2.0, 0.1));
}

float4 fragment(float3 refractVec    : TEXCOORD0,
  	        float3 reflectVec    : TEXCOORD1,
                float3 fresnelTerm   : COLOR0,

           uniform samplerCUBE environmentMaps[2],
           uniform float enableRefraction,
           uniform float enableFresnel) : COLOR
{
    float3 refractColor = texCUBE(environmentMaps[0], refractVec).rgb; 
    float3 reflectColor = texCUBE(environmentMaps[1], reflectVec).rgb; 

    float3 reflectRefract = lerp(refractColor, reflectColor, fresnelTerm);
    
    float3 finalColor = (enableRefraction) ? 
        ((enableFresnel) ? (reflectRefract) : (refractColor)) : 
        ((enableFresnel) ? (reflectColor) : fresnelTerm);

    return float4(finalColor, 1.0);
}
""")

environmentMaps = [ Skybox.cubeMap, Skybox.cubeMap ]

technique = [Render(ZEnable = True,
                    CullMode = CCW,
                    VertexShader = refract.vertex(target=arbvp1),
                    FragmentShader = refract.fragment(target=arbfp1)
                    )]

