

from FX import *
from math import *

def unit( (x,y,z) ):
    n = sqrt(x**2+y**2+z**2)
    return (x/n,y/n,z/n)

class Data:

    def __init__(self,**kwords):
        self.__dict__.update(kwords)

light = Data(position = (0,2,-3),
             color = (1,1,1),
             direction = unit((0,-1,0)),
             constantAtt = 1.0,
             linearAtt = 0.0,
             quadraticAtt = 0.0,
             innerAngle = cos(pi/20.0),
             outerAngle = cos(pi/16.0))

material = Data(ambient = (0.1,0.1,0.1),
                diffuse = (0.5,0.5,0.5),
                specular = (1.0,1.0,1.0),
                shininess = 20.0)

vs = Cg( """
struct SpotLight {
  float3 position;
  float3 color;
  float3 direction;

  float constantAtt;
  float linearAtt;
  float quadraticAtt;

  float innerAngle;
  float outerAngle;
};

struct Material {
  float3 ambient;
  float3 diffuse;
  float3 specular;
  float shininess;
};


float compute_smooth_spotlight( in float3 P, in SpotLight light )
{
  float3 V = normalize( P - float3(0,2,-3) );
  float cosAngle = dot( V, float3(0,1,0) );

  return smoothstep( light.outerAngle,
  		     light.innerAngle,
  		     cosAngle);
}

float compute_sharp_spotlight( in float3 P, in SpotLight light )
{
  float3 V = normalize( P - light.position );
  float cosAngle = dot( V, light.direction );

  if ( cosAngle >= light.innerAngle )
    return 1;
  else 
    return 0;
}

float3 compute_gouraud( in float3 P, 
			in float3 N, 
			in SpotLight light, 
			in Material material )
{
  float3 L = normalize(light.position - P );
  float3 V = normalize( -P );
  float3 H = normalize(L+V);

  float4 l = lit( dot(N,L), dot(N,H), material.shininess );
  
  return ( material.ambient*l.x + 
	   material.diffuse*l.y + 
	   material.specular*l.z );
}


void main(float4 positions : POSITION,
	  float4 normals   : NORMAL,

      out float4 oPosition : POSITION,
      out float4 oColor    : COLOR,
      
  uniform SpotLight light,
  uniform Material material,
  uniform float4x4 ModelView,
  uniform float4x4 ModelViewIT,
  uniform float4x4 Projection)
{
  float4 wPosition = mul( ModelView, positions );
  oPosition = mul( Projection, wPosition );

  float spotlight = compute_smooth_spotlight( wPosition.xyz, light );
  float3 color = compute_gouraud( wPosition.xyz, mul(ModelViewIT,normals).xyz, light, material );
  
  oColor = float4(color * spotlight,1.0);
}
""")      
      
      
technique = [Render(VertexShader = vs())]
