from math import *

EPSILON = 10e-6

def equivalent(a, b):
	if (a < (b + EPSILON)) and (a > (b - EPSILON)):
		return 1
	return 0

class matrix4:
	def __init__(self, *args):
                self.element = [[0 for i in range(4)] for i in range(4)]
		
		if len(args) == 0:
			self.make_identity()
		elif len(args) == 1:
			if type(args[i]) == type(matrix4()):
				self.a__init__(args[i].get_value())
			else:
				self.a__init__(self, args[0])
		elif len(args) == 16:
			self.b__init__(self, args)

	def a__init__(self, r):
		self.set_value(r)

	def b__init__(self, a00, a01, a02, a03, \
			   a10, a11, a12, a13, \
			   a20, a21, a22, a23, \
			   a30, a31, a32, a33):
		self.element[0][0] = a00
		self.element[0][1] = a01
		self.element[0][2] = a02
		self.element[0][3] = a03

		self.element[1][0] = a10
		self.element[1][1] = a11
		self.element[1][2] = a12
		self.element[1][3] = a13

		self.element[2][0] = a20
		self.element[2][1] = a21
		self.element[2][2] = a22
		self.element[2][3] = a23

		self.element[3][0] = a30
		self.element[3][1] = a31
		self.element[3][2] = a32
		self.element[3][3] = a33

	def get_value(self): 
		return self.element[0] + self.element[1] + self.element[2] + self.element[3]

	def get_value(self, m):
		for i in range(4):
			for j in range(4):
				m.element[i][j] = self.element[i][j]	

	def set_value(self, v):
		if type(v) == type([]):
			i = 0
			for y in range(4):
				for x in range(4):
					self.element[y][x] = v[i]
					i = i + 1
		else:
			for y in range(4):
				for x in range(4):
					self.element[y][x] = v

	def make_identity(self):
		self.set_value(0.0)
		self.element[0][0] = 1.0
		self.element[1][1] = 1.0
		self.element[2][2] = 1.0
		self.element[3][3] = 1.0		

	def identity():
		m = matrix4()
		return m

	def set_scale(self, s):
		if type(s) == type([]):
			for x in range(3):
				self.element[x][x] = s[x]
		else:
			self.element[0][0] = s
			self.element[1][1] = s
			self.element[2][2] = s

	def set_translate(self, v):
		for x in range(3):
			self.element[x][3] = v[x]

	def set_row(self, r, v):
		for i in range(4):
			self.element[r][i] = v[i]

	def set_column(self, c, v):
		for i in range(4):
			self.element[i][c] = v[i]

	def get_row(self, c):
		r = []
		for i in range(4):
			r.append(self.element[c][i])
		return r
	
	def get_column(self, c):
		r = []
		for i in range(4):
			r.append(self.element[i][c])
		return r

	def inverse(self):
		minv = matrix4()

		r1 = [0 for i in range(8)]
		r2 = [0 for i in range(8)]
		r3 = [0 for i in range(8)]
		r4 = [0 for i in range(8)]
		s = [r1, r2, r3, r4]

		for i in range(4):
			for j in range(4):
				s[i][j] = self.element[i][j]
				if i == j: s[i][j+4] = 1.0
				else:      s[i][j+4] = 0.0

		scp = [0 for i in range(4)]
		for i in range(4):
			scp[i] = fabs(s[i][0])
			for j in range(1, 4):
				if fabs(s[i][j]) > scp[i]: scp[i] = fabs(s[i][j])
				if scp[i] == 0.0: return minv # singular matrix

		for i in range(4):
			pivot_to = i
			scp_max = fabs(s[i][i] / scp[i])
			for p in range(i+1, 4):
				if fabs(s[p][i] / scp[p]) > scp_max:
					scp_max = fabs(s[p][i] / scp[p])
					pivot_to = p
			if pivot_to != i:
				tmprow = s[i]
				s[i] = s[pivot_to]
				s[pivot_to] = tmprow
				tmpscp = scp[i]
				scp[i] = scp[pivot_to]
				scp[pivot_to] = tmpscp

				
		 	for j in range(i+1,4):
				mji = s[j][i] / s[i][i]
				s[j][i] = 0.0
				for jj in range(i+1,8):
					s[j][jj] = s[j][jj] - mji * s[i][jj]

		if s[3][3] == 0.0: return minv	# Singular
		
		for i in range(3, 0, -1):
			for j in range(i-1, -1, -1):
				mij = s[j][i] / s[i][i]
				for jj in range(j+1, 8):
					s[j][jj] = s[j][jj] - mij * s[i][jj]	
		
		for i in range(4):
			for j in range(4):
				minv.element[i][j] = s[i][j+4] / s[i][i]

		return minv

	def transpose(self):
		m = matrix4()
		for i in range(4): 
			for j in range(4):
				m.element[i][j] = self.element[j][i]

		return m
	
	def mult_right(self, b):
		mt = matrix4(self)
		self.set_value(0.0)
		
		for i in range(4):
			for j in range(4):
				for c in range(4):
					self.element[i][j] = self.element[i][j] + \
						mt.element[i][c] * b.element[c][j]
		return self

	def mult_left(self, b):
		mt = matrix4(self)
		self.set_value(0.0)
		
		for i in range(4):
			for j in range(4):
				for c in range(4):
					self.element[i][j] = self.element[i][j] + \
						b.element[i][c] * mt.element[c][j]
		return self

	def mult_matrix_vec(self, src, dst):
		if len(src) == 3:
			w = src[0] * self.element[3][0] + \
			    src[1] * self.element[3][1] + \
			    src[2] * self.element[3][2] + \
			    self.element[3][3]

			for i in range(3):
				dst[i] = (src[0] * self.element[i][0] + \
					  src[1] * self.element[i][1] + \
					  src[2] * self.element[i][2] + \
					  self.element[i][3]) / w	
		elif len(src) == 4:
			for i in range(4):
				dst[i] = (src[0] * self.element[i][0] + \
                                          src[1] * self.element[i][1] + \
                                          src[2] * self.element[i][2] + \
                                          src[3] * self.element[i][3]) 
			
	def mult_matrix_vec(self, both):
		self.mult_matrix_vec(both, both)

	def mult_vec_matrix(self, src, dst):
		if len(src) == 3:
			w = src[0] * self.element[0][3] + \
			    src[1] * self.element[1][3] + \
			    src[2] * self.element[2][3] + \
			    self.element[3][3]

			for i in range(3):
				dst[i] = (src[0] * self.element[0][i] + \
					  src[1] * self.element[1][i] + \
					  src[2] * self.element[2][i] + \
					  self.element[3][i]) / w	
		elif len(src) == 4:
			for i in range(4):
				dst[i] = (src[0] * self.element[0][i] + \
                                          src[1] * self.element[1][i] + \
                                          src[2] * self.element[2][i] + \
                                          src[3] * self.element[3][i]) 

	def mult_vec_matrix(self, both):
		self.mult_vec_matrix(both, both)

	def mult_matrix_dir(self, src, dst):
		for i in range(3):
			dst[i] = src[0] * self.element[i][0] + \
				 src[1] * self.element[i][1] + \
				 src[2] * self.element[i][2]

	def mult_dir_matrix(self, src, dst):
		for i in range(3):
			dst[i] = src[0] * self.element[0][i] + \
				 src[1] * self.element[1][i] + \
				 src[2] * self.element[2][i]

	def mult_matrix_dir(self, both): self.mult_matrix_dir(both, both)
	def mult_dir_matrix(self, both): self.mult_dir_matrix(both, both)

class quaternion:
	def __init__(self, *args):
		self.q = [0, 0, 0, 0]
		self.counter = 0

		if len(args) == 0:
			self.set(self.identity())
		elif len(args) == 1:
			self.set_vec(args[0])
		elif len(args) == 4:
			self.set_four(args[0], args[1], args[2], args[3])
		elif len(args) == 2:
			self.set_value(args[0], args[1])

	def set(self, quat):
		self.q = [quat.q[i] for i in range(4)]

	def set_vec(self, v):
		self.q = [v[i] for i in range(4)]
	
	def set_four(self, a, b, c, d):
		self.q = [a, b, c, d]

	def set_value(self, axis, angle):
		sqnorm = axis[0]*axis[0] + axis[1]*axis[1] + axis[2]*axis[2]
		if sqnorm <= EPSILON:
			self.q = [0, 0, 0, 1]
		else:
			angle = angle * 0.5
			sin_angle = sin(angle)
			if not equivalent(sqnorm, 1.0):
				sin_angle = sin_angle / sqrt(sqnorm)
			self.q[0] = sin_angle * axis[0]
			self.q[1] = sin_angle * axis[1]
			self.q[2] = sin_angle * axis[2]
			self.q[3] = cos(angle)

	def get_value_q(self): return [self.q[i] for i in range(4)]
	def get_value(self):
		radians = acos(self.q[3]) * 2.0
		axis = [0, 0, 0]
		if radians == 0.0:
			axis = [0.0, 0.0, 1.0]
		else:
			axis[0] = self.q[0]
			axis[1] = self.q[1]
			axis[2] = self.q[2]
			alen = sqrt(axis[0]*axis[0]+axis[1]*axis[1]+axis[2]*axis[2])
			axis = [axis[i]/alen for i in range(3)]
		return axis, radians
	def inverse(self):
		r = quaternion()
		r.set(self)
		r.invert()
		return r
	def invert(self):
		return self.conjugate()	
	def conjugate(self):
		self.q = [-self.q[0], -self.q[1], -self.q[2], self.q[3]]
		return self
	def identity(self):
		return quaternion([0.0, 0.0, 0.0], 1.0)

	def mult_right(self, qr):
		ql = quaternion()
		ql.set(self)
		self.q[3] = ql.q[3] * qr.q[3] - ql.q[0] * qr.q[0] - ql.q[1] * qr.q[1] - ql.q[2] * qr.q[2]
		self.q[0] = ql.q[3] * qr.q[0] + ql.q[0] * qr.q[3] + ql.q[1] * qr.q[2] - ql.q[2] * qr.q[1]
		self.q[1] = ql.q[3] * qr.q[1] + ql.q[1] * qr.q[3] + ql.q[2] * qr.q[0] - ql.q[0] * qr.q[2]
		self.q[2] = ql.q[3] * qr.q[2] + ql.q[2] * qr.q[3] + ql.q[0] * qr.q[1] - ql.q[1] * qr.q[0]

		self.counter = self.counter + qr.counter + 1
		self.counter_normalize()
		return self	
	def counter_normalize(self):
		if self.counter > 64:
			self.normalize()
	def normalize(self):
		rnorm = 1.0 / sqrt(self.q[3] * self.q[3] + self.q[0] * self.q[0] \
				+ self.q[1] * self.q[1] + self.q[2] * self.q[2])
		if equivalent(rnorm, 0.0): return
		self.q = [self.q[i] * rnorm for i in range(4)]
		self.counter = 0
